@@ -1,7 +1,7 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/pdfgen/pdfimages.py
-__version__=''' $Id$ '''
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/src/reportlab/pdfgen/pdfimages.py
+__version__='3.3.0'
 __doc__="""
 Image functionality sliced out of canvas.py for generalization
 """
@@ -11,7 +11,7 @@ import reportlab
 from reportlab import rl_config
 from reportlab.pdfbase import pdfutils
 from reportlab.pdfbase import pdfdoc
-from reportlab.lib.utils import import_zlib, haveImages, getBytesIO, isStr
+from reportlab.lib.utils import haveImages, isStr
 from reportlab.lib.rl_accel import fp_str, asciiBase85Encode
 from reportlab.lib.boxstuff import aspectRatioFix
 
@@ -74,8 +74,6 @@ class PDFImage:
     def cache_imagedata(self):
         image = self.image
         if not pdfutils.cachedImageExists(image):
-            zlib = import_zlib()
-            if not zlib: return
             if not haveImages: return
             pdfutils.cacheImageFile(image)
 
@@ -87,14 +85,13 @@ class PDFImage:
         return imagedata
 
     def PIL_imagedata(self):
+        import zlib
         image = self.image
         if image.format=='JPEG':
             fp=image.fp
             fp.seek(0)
             return self._jpg_imagedata(fp)
         self.source = 'PIL'
-        zlib = import_zlib()
-        if not zlib: return
 
         bpc = 8
         # Use the colorSpace in the image
@@ -122,8 +119,8 @@ class PDFImage:
         imagedata=['BI /W %d /H %d /BPC %d /CS /%s /F [%s/Fl] ID' % (imgwidth, imgheight, bpc, colorSpace, rl_config.useA85 and '/A85 ' or '')]
 
         #use a flate filter and, optionally, Ascii Base 85 to compress
-        raw = myimage.tostring()
-        rowstride = (imgwidth*bpc*bpp+7)/8
+        raw = (myimage.tobytes if hasattr(myimage,'tobytes') else myimage.tostring)()
+        rowstride = (imgwidth*bpc*bpp+7)>>3
         assert len(raw) == rowstride*imgheight, "Wrong amount of data for image"
         data = zlib.compress(raw)    #this bit is very fast...
         if rl_config.useA85:
@@ -157,19 +154,14 @@ class PDFImage:
             else:
                 imagedata, imgwidth, imgheight = self.non_jpg_imagedata(image)
         else:
-            import sys
-            if sys.platform[0:4] == 'java':
-                #jython, PIL not available
-                imagedata, imgwidth, imgheight = self.JAVA_imagedata()
-            else:
-                imagedata, imgwidth, imgheight = self.PIL_imagedata()
+            imagedata, imgwidth, imgheight = self.PIL_imagedata()
         self.imageData = imagedata
         self.imgwidth = imgwidth
         self.imgheight = imgheight
         self.width = self.width or imgwidth
         self.height = self.height or imgheight
 
-    def drawInlineImage(self, canvas, preserveAspectRatio=False,anchor='sw'):
+    def drawInlineImage(self, canvas, preserveAspectRatio=False,anchor='sw', anchorAtXY=False, showBoundary=False):
         """Draw an Image into the specified rectangle.  If width and
         height are omitted, they are calculated from the image size.
         Also allow file names as well as images.  This allows a
@@ -177,7 +169,7 @@ class PDFImage:
         width = self.width
         height = self.height
         if width<1e-6 or height<1e-6: return False
-        x,y,self.width,self.height, scaled = aspectRatioFix(preserveAspectRatio,anchor,self.x,self.y,width,height,self.imgwidth,self.imgheight)
+        x,y,self.width,self.height, scaled = aspectRatioFix(preserveAspectRatio,anchor,self.x,self.y,width,height,self.imgwidth,self.imgheight,anchorAtXY)
         # this says where and how big to draw it
         if not canvas.bottomup: y = y+height
         canvas._code.append('q %s 0 0 %s cm' % (fp_str(self.width), fp_str(self.height, x, y)))
@@ -185,6 +177,8 @@ class PDFImage:
         for line in self.imageData:
             canvas._code.append(line)
         canvas._code.append('Q')
+        if showBoundary:
+            canvas.drawBoundary(showBoundary,x,y,width,height)
         return True
 
     def format(self, document):
@@ -211,7 +205,6 @@ if __name__=='__main__':
     assert os.path.isfile(srcfile), 'image not found'
     pdfdoc.LongFormat = 1
     img = PDFImage(srcfile, 100, 100)
-    import pprint
     doc = pdfdoc.PDFDocument()
     print('source=',img.source)
     print(img.format(doc))
