@@ -15,7 +15,6 @@ pageCompression
 useA85
 defaultPageSize
 defaultImageCaching
-ZLIB_WARNINGS
 warnOnMissingFontGlyphs
 verbose
 showBoundary
@@ -67,7 +66,8 @@ reserveTTFNotdef
 documentLang
 encryptionStrength
 trustedHosts
-trustedSchemes'''.split())
+trustedSchemes
+renderPMBackend'''.split())
 
 allowTableBoundsErrors =    1 # set to 0 to die on too large elements in tables in debug (recommend 1 for production use)
 shapeChecking =             1
@@ -77,7 +77,6 @@ pageCompression =           1                       # default page compression m
 useA85 =                    1                       #set to 0 to disable Ascii Base 85 stream filters
 defaultPageSize =           'A4'                    #default page size
 defaultImageCaching =       0                       #set to zero to remove those annoying cached images
-ZLIB_WARNINGS =             1
 warnOnMissingFontGlyphs =   0                       #if 1, warns of each missing glyph
 verbose =                   0
 showBoundary =              0                       # turns on and off boundary behaviour in Drawing
@@ -161,6 +160,7 @@ trustedHosts=None                                   #set to a list of trusted fo
                                                     #allowed. In environment use a comma separated string.
 trustedSchemes=['file', 'rml', 'data', 'https',     #these url schemes are trusted
                 'http', 'ftp']
+renderPMBackend='_renderPM'                         #or 'rlPyCairo' if available
 
 # places to look for T1Font information
 T1SearchPath =  (
