@@ -3,7 +3,8 @@
 """Tests for reportlab.lib.utils
 """
 __version__='3.3.0'
-from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, printLocation
+from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, printLocation, mockUrlRead
+from unittest.mock import patch
 setOutDir(__name__)
 import os, time, sys
 import reportlab
@@ -11,8 +12,9 @@ from reportlab import rl_config
 import unittest
 from reportlab.lib import colors
 from reportlab.lib.utils import recursiveImport, recursiveGetAttr, recursiveSetAttr, rl_isfile, \
-                                isCompactDistro, isPy3, isPyPy, TimeStamp, rl_get_module, \
-                                recursiveGetAttr, recursiveSetAttr, recursiveDelAttr
+                                isCompactDistro, isPyPy, TimeStamp, rl_get_module, \
+                                recursiveGetAttr, recursiveSetAttr, recursiveDelAttr, \
+                                asUnicode, asUnicodeEx, asBytes
 
 def _rel_open_and_read(fn):
     from reportlab.lib.utils import open_and_read
@@ -38,7 +40,7 @@ class ImporterTestCase(unittest.TestCase):
         _testmodulename = os.path.join(cls._tempdir,'test_module_%s.py' % s)
         with open(_testmodulename,'w') as f:
             f.write('__all__=[]\nvalue=%s\n' % repr(cls._value))
-        if sys.platform=='darwin' and isPy3:
+        if sys.platform=='darwin':
             time.sleep(0.3)
         cls._testmodulename = os.path.splitext(os.path.basename(_testmodulename))[0]
 
@@ -47,6 +49,9 @@ class ImporterTestCase(unittest.TestCase):
         from shutil import rmtree
         rmtree(cls._tempdir,1)
 
+    def myAssertRaisesRegex(self,*args,**kwds):
+        return self.assertRaisesRegex(*args,**kwds)
+
     def test1(self):
         "try stuff known to be in the path"
         m1 = recursiveImport('reportlab.pdfgen.canvas')
@@ -113,6 +118,7 @@ class ImporterTestCase(unittest.TestCase):
         "test open and read of a relative file: URL"
         b = _rel_open_and_read('file:../docs/images/Edit_Prefs.gif')
 
+    @patch('reportlab.lib.utils.rlUrlRead',mockUrlRead)
     def test9(self):
         "test open and read of an http: URL"
         from reportlab.lib.utils import open_and_read
@@ -120,8 +126,9 @@ class ImporterTestCase(unittest.TestCase):
 
     def test10(self):
         "test open and read of a simple relative file"
-        from reportlab.lib.utils import open_and_read, getBytesIO
-        b = getBytesIO(_rel_open_and_read('../docs/images/Edit_Prefs.gif'))
+        from io import BytesIO
+        from reportlab.lib.utils import open_and_read
+        b = BytesIO(_rel_open_and_read('../docs/images/Edit_Prefs.gif'))
         b = open_and_read(b)
 
     def test11(self):
@@ -154,9 +161,7 @@ class ImporterTestCase(unittest.TestCase):
             self.fail("Imported a buggy module")
         except Exception as e:
             self.assertIn(("integer division by zeroException raised while importing 'unimportable': integer division by zero"
-                            if isPyPy
-                            else ('division by zero' if isPy3
-                                 else 'integer division or modulo by zero'))
+                            if isPyPy else 'division by zero')
                         ,str(e))
     def test14(self):
         "test the TimeStamp behaviour"
@@ -208,6 +213,18 @@ class ImporterTestCase(unittest.TestCase):
         ff.search()
         ff.getFamilyNames()
 
+    def test17(self):
+        self.assertEqual(asUnicode(u'abc'),u'abc')
+        self.assertEqual(asUnicode(b'abc'),u'abc')
+        self.assertRaises(AttributeError,asUnicode,['abc'])
+        self.myAssertRaisesRegex(AttributeError,r"asUnicode\(.*'list' object has no attribute 'decode'", asUnicode,['abc'])
+        self.assertEqual(asUnicodeEx(u'abc'),u'abc')
+        self.assertEqual(asUnicodeEx(b'abc'),u'abc')
+        self.assertEqual(asUnicodeEx(123),u'123')
+        self.assertEqual(asBytes(u'abc'),b'abc')
+        self.assertEqual(asBytes(b'abc'),b'abc')
+        self.assertRaises(AttributeError,asBytes,['abc'])
+        self.myAssertRaisesRegex(AttributeError,"asBytes\(.*'list' object has no attribute 'encode'", asBytes,['abc'])
 
 class RaccessTest:
     l = [1, 2]
